/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <riq@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
#pragma once

#include "../common/common.h"

#ifdef __cplusplus
extern "C" {
#endif

/// Place the regular armies
TEG_STATUS ai_fichas(int cant);

/// Place continent assigned armies
TEG_STATUS ai_fichasc(int cant, int conts);

/// Calls all the strategies of the robots to place armies
TEG_STATUS ai_fichas_calc_puntaje(int p);

#ifdef __cplusplus
}
#endif
