/* Tenes Empanadas Graciela
 *
 * Copyright (C) 2000 Ricardo Quesada
 *
 * Author: Ricardo Calixto Quesada <rquesada@core-sdi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; only version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
/*
 * general includes for the TEG client
 */

#ifndef __TEGC_CLIENT_H
#define __TEGC_CLIENT_H

#include "player.h"
#include "globals.h"
#include "misc.h"
#include "helper.h"
#include "inputfd.h"
#include "outputfd.h"
#include "fichas.h"
#include "attack.h"
#include "reagrupe.h"
#include "ejer2.h"
#include "canje.h"

#endif /* __TEGC_CLIENT_H */
